@@ -2,7 +2,7 @@
  *
  * rasqal_engine.c - Rasqal query engine
  *
- * $Id: rasqal_engine.c,v 1.126 2005/03/21 18:25:10 cmdjb Exp $
+ * $Id: rasqal_engine.c,v 1.156 2005/06/08 16:33:16 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -137,6 +137,9 @@ rasqal_engine_sequence_has_qname(raptor_sequence *seq)
 int
 rasqal_engine_query_constraints_has_qname(rasqal_query* rq) 
 {
+  if(!rq->query_graph_pattern)
+    return 0;
+  
   return rasqal_engine_graph_pattern_constraints_has_qname(rq->query_graph_pattern);
 }
 
@@ -213,6 +216,9 @@ rasqal_engine_build_constraints_expression(rasqal_graph_pattern* gp)
 {
   rasqal_expression* newe=NULL;
   int i;
+
+  if(!gp)
+    return 1;
   
   /* build constraints in sub graph patterns */
   if(gp->graph_patterns) {
@@ -241,31 +247,92 @@ rasqal_engine_build_constraints_expression(rasqal_graph_pattern* gp)
 }
 
 
+static void
+rasqal_engine_convert_blank_node_to_anonymous_variable(rasqal_query *rq,
+                                                       rasqal_literal *l) {
+  rasqal_variable* v;
+  
+  v=rasqal_new_variable_typed(rq, 
+                              RASQAL_VARIABLE_TYPE_ANONYMOUS,
+                              (unsigned char*)l->string, NULL);
+
+  /* Convert the blank node literal into a variable literal */
+  l->string=NULL;
+
+  l->type=RASQAL_LITERAL_VARIABLE;
+  l->value.variable=v;
+}
+
+
+static int
+rasqal_engine_build_anonymous_variables(rasqal_query* rq)
+{
+  int i;
+  raptor_sequence *s=rq->triples;
+  
+  for(i=0; i < raptor_sequence_size(s); i++) {
+    rasqal_triple* t=(rasqal_triple*)raptor_sequence_get_at(s, i);
+    if(t->subject->type == RASQAL_LITERAL_BLANK)
+      rasqal_engine_convert_blank_node_to_anonymous_variable(rq, t->subject);
+    if(t->predicate->type == RASQAL_LITERAL_BLANK)
+      rasqal_engine_convert_blank_node_to_anonymous_variable(rq, t->predicate);
+    if(t->object->type == RASQAL_LITERAL_BLANK)
+      rasqal_engine_convert_blank_node_to_anonymous_variable(rq, t->object);
+  }
+
+  return 0;
+}
+
+
+static int
+rasqal_engine_expand_wildcards(rasqal_query* rq)
+{
+  int i;
+  raptor_sequence *s;
+  int rc=0;
+
+  if(!rq->wildcard)
+    return rc;
+  
+  switch(rq->verb) {
+    case  RASQAL_QUERY_VERB_SELECT:
+    /* If 'SELECT *' was given, make the selects be a list of all variables */
+      rq->selects=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
+      
+      for(i=0; i< rq->variables_count; i++)
+        raptor_sequence_push(rq->selects, raptor_sequence_get_at(rq->variables_sequence, i));
+      break;
+      
+    case RASQAL_QUERY_VERB_CONSTRUCT:
+      /* If 'CONSTRUCT *' was given, make the constructs be all triples */
+      rq->constructs=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
+      s=((rasqal_query*)rq)->triples;
+      
+      for(i=0; i < raptor_sequence_size(s); i++) {
+        rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(s, i);
+        raptor_sequence_push(rq->constructs, rasqal_new_triple_from_triple(t));
+      }
+      break;
+
+    case RASQAL_QUERY_VERB_UNKNOWN:
+    case RASQAL_QUERY_VERB_DESCRIBE:
+    case RASQAL_QUERY_VERB_ASK:
+    default:
+      rasqal_query_error(rq, "Cannot use wildcard * with query verb ",
+                         rasqal_query_verb_as_string(rq->verb));
+      rc=1;
+      break;
+  }
+
+  return rc;
+}
+
+
 int
 rasqal_engine_assign_variables(rasqal_query* rq)
 {
   int i;
-
-  /* If 'SELECT *' was given, make the selects be a list of all variables */
-  if(rq->select_all) {
-    rq->selects=raptor_new_sequence(NULL, (raptor_sequence_print_handler*)rasqal_variable_print);
-    
-    for(i=0; i< rq->variables_count; i++)
-      raptor_sequence_push(rq->selects, raptor_sequence_get_at(rq->variables_sequence, i));
-  }
-
-  /* If 'CONSTRUCT *' was given, make the constructs be all triples */
-  if(rq->construct_all) {
-    raptor_sequence *s;
-    
-    rq->constructs=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_triple, (raptor_sequence_print_handler*)rasqal_triple_print);
-    s=((rasqal_query*)rq)->triples;
-
-    for(i=0; i < raptor_sequence_size(s); i++) {
-      rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(s, i);
-      raptor_sequence_push(rq->constructs, rasqal_new_triple_from_triple(t));
-    }
-  }
+  int offset;
 
   if(rq->selects)
     rq->select_variables_count=raptor_sequence_size(rq->selects);
@@ -275,14 +342,24 @@ rasqal_engine_assign_variables(rasqal_query* rq)
     rq->binding_values=(rasqal_literal**)RASQAL_MALLOC(rasqal_literals,sizeof(rasqal_literal*)*(rq->select_variables_count+1));
   }
   
-  rq->variables=(rasqal_variable**)RASQAL_MALLOC(varrary, sizeof(rasqal_variable*)*rq->variables_count);
-  rq->variables_declared_in=(int*)RASQAL_CALLOC(intarray, sizeof(int), rq->variables_count+1);
+  rq->variables=(rasqal_variable**)RASQAL_MALLOC(varrary, sizeof(rasqal_variable*)*(rq->variables_count + rq->anon_variables_count));
+  rq->variables_declared_in=(int*)RASQAL_CALLOC(intarray, sizeof(int), rq->variables_count + rq->anon_variables_count + 1);
 
+  offset=0;
   for(i=0; i< rq->variables_count; i++) {
-    rq->variables_declared_in[i]= -1;
-    rq->variables[i]=(rasqal_variable*)raptor_sequence_get_at(rq->variables_sequence, i);
-    if(i< rq->select_variables_count)
-      rq->variable_names[i]=rq->variables[i]->name;
+    rq->variables_declared_in[offset]= -1;
+    rq->variables[offset]=(rasqal_variable*)raptor_sequence_get_at(rq->variables_sequence, i);
+    if(i < rq->select_variables_count)
+      rq->variable_names[offset]=rq->variables[offset]->name;
+    offset++;
+  }
+
+  for(i=0; i< rq->anon_variables_count; i++) {
+    rq->variables_declared_in[offset]= -1;
+    rq->variables[offset]=(rasqal_variable*)raptor_sequence_get_at(rq->anon_variables_sequence, i);
+    /* only now can we make this offset absolute into the full list of vars */
+    rq->variables[offset]->offset += rq->variables_count;
+    offset++;
   }
 
   if(rq->variable_names) {
@@ -309,7 +386,7 @@ static rasqal_triples_source_factory Triples_Source_Factory;
  * called with the user data for some query and rasqal_triples_source.
  * 
  **/
-RASQAL_API void
+void
 rasqal_set_triples_source_factory(void (*register_fn)(rasqal_triples_source_factory *factory), void* user_data) {
   Triples_Source_Factory.user_data=user_data;
   register_fn(&Triples_Source_Factory);
@@ -377,12 +454,21 @@ static rasqal_triples_match*
 rasqal_new_triples_match(rasqal_query *query, void *user_data,
                          rasqal_triple_meta *m, rasqal_triple *t)
 {
+  rasqal_triples_match* rtm;
+
   if(!query->triples_source)
     return NULL;
-  
-  return query->triples_source->new_triples_match(query->triples_source,
-                                                  query->triples_source->user_data,
-                                                  m, t);
+
+  rtm=(rasqal_triples_match *)RASQAL_CALLOC(rasqal_triples_match, sizeof(rasqal_triples_match), 1);
+  if(query->triples_source->init_triples_match(rtm,
+                                               query->triples_source,
+                                               query->triples_source->user_data,
+                                               m, t)) {
+    RASQAL_FREE(rasqal_triples_match, rtm);
+    rtm=NULL;
+  }
+
+  return rtm;
 }
 
 
@@ -443,15 +529,23 @@ rasqal_reset_triple_meta(rasqal_triple_meta* m)
     resets++;
   }
   
-  if(m->triples_match) {
-    rasqal_free_triples_match(m->triples_match);
-    m->triples_match=NULL;
-  }
-
   return resets;
 }
 
 
+static RASQAL_INLINE void
+rasqal_query_graph_pattern_build_declared_in_variable(rasqal_query* query,
+                                                      rasqal_variable *v,
+                                                      int col)
+{
+  if(!v)
+    return;
+  
+  if(query->variables_declared_in[v->offset] < 0)
+    query->variables_declared_in[v->offset]=col;
+}
+
+
 /**
  * rasqal_query_graph_pattern_build_declared_in - Mark where variables are first declared in a graph_pattern
  * @query; the &rasqal_query to find the variables in
@@ -477,25 +571,20 @@ rasqal_query_graph_pattern_build_declared_in(rasqal_query* query,
     
   for(col=gp->start_column; col <= gp->end_column; col++) {
     rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(gp->triples, col);
-    rasqal_variable* v;
-    
-    if((v=rasqal_literal_as_variable(t->subject)) &&
-       query->variables_declared_in[v->offset] < 0)
-      query->variables_declared_in[v->offset]=col;
-    
-    if((v=rasqal_literal_as_variable(t->predicate)) &&
-       query->variables_declared_in[v->offset] < 0)
-      query->variables_declared_in[v->offset]=col;
-    
-    if((v=rasqal_literal_as_variable(t->object)) &&
-       query->variables_declared_in[v->offset] < 0)
-      query->variables_declared_in[v->offset]=col;
-    
-    if(t->origin &&
-       (v=rasqal_literal_as_variable(t->origin)) &&
-       query->variables_declared_in[v->offset] < 0)
-      query->variables_declared_in[v->offset]=col;
-    
+
+    rasqal_query_graph_pattern_build_declared_in_variable(query,
+                                                          rasqal_literal_as_variable(t->subject),
+                                                          col);
+    rasqal_query_graph_pattern_build_declared_in_variable(query,
+                                                          rasqal_literal_as_variable(t->predicate),
+                                                          col);
+    rasqal_query_graph_pattern_build_declared_in_variable(query,
+                                                          rasqal_literal_as_variable(t->object),
+                                                          col);
+    if(t->origin)
+      rasqal_query_graph_pattern_build_declared_in_variable(query,
+                                                            rasqal_literal_as_variable(t->origin),
+                                                            col);
   }
   
 }
@@ -512,6 +601,9 @@ rasqal_query_build_declared_in(rasqal_query* query)
   int i;
   rasqal_graph_pattern *gp=query->query_graph_pattern;
 
+  if(!gp)
+    return;
+  
   rasqal_query_graph_pattern_build_declared_in(query, gp);
 
   for(i=0; i< query->variables_count; i++) {
@@ -525,7 +617,7 @@ rasqal_query_build_declared_in(rasqal_query* query)
 #endif
     } else 
       rasqal_query_warning(query, 
-                           "Variable %s was selected but is unused in query.\n", 
+                           "Variable %s was selected but is unused in the query.", 
                            v->name);
   }
 
@@ -561,7 +653,7 @@ rasqal_graph_pattern_get_next_match(rasqal_query *query,
       gp->column--;
       rc= -1;
       return rc;
-    } else if (t->flags & RASQAL_TRIPLE_FLAGS_EXACT) {
+    } else if (m->is_exact) {
       /* exact triple match wanted */
       if(!rasqal_triples_source_triple_present(query->triples_source, t)) {
         /* failed */
@@ -628,7 +720,7 @@ rasqal_graph_pattern_get_next_match(rasqal_query *query,
       /* Done all conjunctions */ 
       
       /* exact match, so column must have ended */
-      if(t->flags & RASQAL_TRIPLE_FLAGS_EXACT)
+      if(m->is_exact)
         gp->column--;
 
       /* return with result (rc is 1) */
@@ -653,13 +745,17 @@ rasqal_graph_pattern_get_next_match(rasqal_query *query,
 int
 rasqal_engine_prepare(rasqal_query *query)
 {
-  int i;
-
   if(!query->triples)
     return 1;
   
   if(!query->variables) {
-    /* Expand 'SELECT *' and create the query->variables array */
+
+    rasqal_engine_build_anonymous_variables(query);
+
+    /* Expand 'SELECT *' and 'CONSTRUCT *' */
+    rasqal_engine_expand_wildcards(query);
+
+    /* create the query->variables array */
     rasqal_engine_assign_variables(query);
 
     rasqal_query_build_declared_in(query);
@@ -667,17 +763,6 @@ rasqal_engine_prepare(rasqal_query *query)
     rasqal_engine_build_constraints_expression(query->query_graph_pattern);
   }
 
-  for(i=0; i < raptor_sequence_size(query->triples); i++) {
-    rasqal_triple *t=(rasqal_triple*)raptor_sequence_get_at(query->triples, i);
-    
-    t->flags |= RASQAL_TRIPLE_FLAGS_EXACT;
-    if(rasqal_literal_as_variable(t->predicate) ||
-       rasqal_literal_as_variable(t->subject) ||
-       rasqal_literal_as_variable(t->object))
-      t->flags &= ~RASQAL_TRIPLE_FLAGS_EXACT;
-  }
-
-
   return 0;
 }
 
@@ -698,21 +783,40 @@ rasqal_engine_execute_init(rasqal_query* query, rasqal_query_results *results)
     }
   }
 
-  gp=query->query_graph_pattern;
-
-  rasqal_graph_pattern_init(gp);
-  
   query->abort=0;
   query->result_count=0;
   query->finished=0;
   query->failed=0;
   
-  if(gp->graph_patterns && raptor_sequence_size(gp->graph_patterns))
-    gp->current_graph_pattern= 0;
-  else
-    /* FIXME - no graph patterns in query */
-    gp->current_graph_pattern= -1;
+  /* FIXME.  This is hack.  If the structure is a single GP with no sub-GPs
+   * then make a new top graph pattern so the query engine always
+   * sees a sequence of graph patterns at the top.  It should
+   * operate fine on a graph pattern with just triples but the 
+   * engine doesn't do this yet.
+   */
+  if(query->query_graph_pattern) {
+    if(query->query_graph_pattern->triples) {
+      raptor_sequence *seq;
+
+      seq=raptor_new_sequence((raptor_sequence_free_handler*)rasqal_free_graph_pattern, (raptor_sequence_print_handler*)rasqal_graph_pattern_print);
+      raptor_sequence_push(seq, query->query_graph_pattern);
+      
+      query->query_graph_pattern=rasqal_new_graph_pattern_from_sequence(query, seq, RASQAL_GRAPH_PATTERN_OPERATOR_GROUP);
+
+#ifdef RASQAL_DEBUG
+      RASQAL_DEBUG1("Restructured top level single graph pattern to be a sequence of GPs, now:\n");
+      rasqal_query_print(query, stderr);
+#endif
+    }
+    
+  }
   
+
+  gp=query->query_graph_pattern;
+
+  if(gp)
+    rasqal_graph_pattern_init(gp);
+    
   return 0;
 }
 
@@ -1021,6 +1125,17 @@ rasqal_engine_do_optional_step(rasqal_query *query,
    return STEP_SEARCHING;
  }
  
+
+  if(outergp->constraints_expression) {
+    step=rasqal_engine_check_constraint(query, outergp);
+    if(step != STEP_GOT_MATCH) {
+      /* The constraint failed or we have an error - no bindings count */
+      query->new_bindings_count=0;
+      return STEP_SEARCHING;
+    }
+  }
+
+
  /* is the last graph pattern so we have a solution */
 
   RASQAL_DEBUG1("Got match\n");
@@ -1076,7 +1191,7 @@ rasqal_engine_get_next_result(rasqal_query *query)
     
     RASQAL_DEBUG3("Handling graph_pattern %d %s\n",
                   outergp->current_graph_pattern,
-                  (gp->flags & RASQAL_PATTERN_FLAGS_OPTIONAL) ? "(OPTIONAL)" : "");
+                  rasqal_graph_pattern_operator_as_string(gp->op));
 
     if(gp->graph_patterns) {
       /* FIXME - sequence of graph_patterns not implemented, finish */
@@ -1086,7 +1201,7 @@ rasqal_engine_get_next_result(rasqal_query *query)
     }
 
     gp->matched=0;
-    optional_step=(gp->flags & RASQAL_PATTERN_FLAGS_OPTIONAL);
+    optional_step=(gp->op == RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL);
     
     if(optional_step)
       step=rasqal_engine_do_optional_step(query, outergp, gp);
@@ -1129,7 +1244,6 @@ rasqal_engine_get_next_result(rasqal_query *query)
     }
 
     /* Got a valid result */
-    query->result_count++;
 #ifdef RASQAL_DEBUG
     RASQAL_DEBUG1("Returning solution[");
     for(i=0; i< query->select_variables_count; i++) {
@@ -1193,3 +1307,170 @@ rasqal_engine_assign_binding_values(rasqal_query *query)
   for(i=0; i< query->select_variables_count; i++)
     query->binding_values[i]=query->variables[i]->value;
 }
+
+
+void
+rasqal_engine_make_basic_graph_pattern(rasqal_graph_pattern *gp)
+{
+  int merge_gp_ok=0;
+  int i;
+  int size;
+
+#if RASQAL_DEBUG > 1
+  printf("rasqal_engine_make_basic_graph_pattern: Checking graph pattern %p:\n  ", gp);
+  rasqal_graph_pattern_print(gp, stdout);
+  fputs("\n", stdout);
+  RASQAL_DEBUG3("Columns %d to %d\n", gp->start_column, gp->end_column);
+#endif
+    
+  if(!gp->graph_patterns) {
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG2("Ending graph patterns %p - no sub-graph patterns\n", gp);
+#endif
+    return;
+  }
+
+  if(gp->op != RASQAL_GRAPH_PATTERN_OPERATOR_GROUP &&
+     gp->op != RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL) {
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG3("Ending graph patterns %p - operator %s\n", gp,
+                  rasqal_graph_pattern_operator_as_string(gp->op));
+#endif
+    return;
+  }
+
+  size=raptor_sequence_size(gp->graph_patterns);
+#if RASQAL_DEBUG > 1
+  RASQAL_DEBUG3("Doing %d sub-graph patterns of %p\n", size, gp);
+#endif
+  for(i=0; i < size; i++) {
+    rasqal_graph_pattern *sgp=(rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+    rasqal_engine_make_basic_graph_pattern(sgp);
+  }
+#if RASQAL_DEBUG > 1
+  RASQAL_DEBUG2("Sub-graph patterns of %p done\n", gp);
+#endif
+
+
+  if(size == 1) {
+    merge_gp_ok=1;
+    goto merge_check_done;
+  }
+
+  /* check if ALL sub-graph patterns are basic graph patterns
+   * and either:
+   * 1) a single triple
+   * 2) a single constraint
+   */
+  for(i=0; i < raptor_sequence_size(gp->graph_patterns); i++) {
+    rasqal_graph_pattern *sgp=(rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+    
+    if(sgp->op != RASQAL_GRAPH_PATTERN_OPERATOR_BASIC) {
+#if RASQAL_DEBUG > 1
+      RASQAL_DEBUG3("Found %s sub-graph pattern %p\n",
+                    rasqal_graph_pattern_operator_as_string(sgp->op), 
+                    sgp);
+#endif
+      merge_gp_ok=0;
+      break;
+    }
+    
+    /* not ok if there are >1 triples */
+    if(sgp->triples && (sgp->end_column-sgp->end_column+1) > 1) {
+#if RASQAL_DEBUG > 1
+      RASQAL_DEBUG2("Found >1 triples in sub-graph pattern %p\n", sgp);
+#endif
+      merge_gp_ok=0;
+      break;
+    }
+    
+    /* not ok if there >1 constraints */
+    if(sgp->constraints && raptor_sequence_size(sgp->constraints) != 1) {
+#if RASQAL_DEBUG > 1
+      RASQAL_DEBUG2("Found >1 constraints in sub-graph pattern %p\n", sgp);
+#endif
+      merge_gp_ok=0;
+      break;
+    }
+    
+    /* not ok if there are triples and constraints */
+    if(sgp->triples && sgp->constraints) {
+#if RASQAL_DEBUG > 1
+      RASQAL_DEBUG2("Found triples and constraints in sub-graph pattern %p\n", sgp);
+#endif
+      merge_gp_ok=0;
+      break;
+    }
+    
+    /* was at least 1 OK sub graph-pattern */
+    merge_gp_ok=1;
+  }
+
+  merge_check_done:
+  
+  if(merge_gp_ok) {
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG2("OK to merge sub-graph patterns of %p\n", gp);
+
+    RASQAL_DEBUG3("Initial columns %d to %d\n", gp->start_column, gp->end_column);
+#endif
+    while(raptor_sequence_size(gp->graph_patterns) > 0) {
+      rasqal_graph_pattern *sgp=(rasqal_graph_pattern*)raptor_sequence_unshift(gp->graph_patterns);
+      if(sgp->triples) {
+        int start_c=sgp->start_column;
+        int end_c=sgp->end_column;
+        
+        /* if this is our first triple, save a free/alloc */
+        gp->triples=sgp->triples;
+        sgp->triples=NULL;
+
+        if((gp->start_column < 0) || start_c < gp->start_column)
+          gp->start_column=start_c;
+        if((gp->end_column < 0) || end_c > gp->end_column)
+          gp->end_column=end_c;
+
+#if RASQAL_DEBUG > 1
+        RASQAL_DEBUG3("Moved triples from columns %d to %d\n", start_c, end_c);
+        RASQAL_DEBUG3("Columns now %d to %d\n", gp->start_column, gp->end_column);
+#endif
+      }
+     
+      if(sgp->constraints) {
+        while(raptor_sequence_size(sgp->constraints) > 0) {
+          rasqal_expression* e=(rasqal_expression*)raptor_sequence_pop(sgp->constraints);
+          rasqal_graph_pattern_add_constraint(gp, e);
+#if RASQAL_DEBUG > 1
+          RASQAL_DEBUG1("Moved a constraint\n");
+#endif
+        }
+      }
+      
+      rasqal_free_graph_pattern(sgp);
+    }
+
+    /* Delete any evidence of sub graph patterns */
+    raptor_free_sequence(gp->graph_patterns);
+    gp->graph_patterns=NULL;
+
+    /* Update operator GROUP => BASIC, but do not change OPTIONAL */
+    if(gp->op == RASQAL_GRAPH_PATTERN_OPERATOR_GROUP)
+      gp->op=RASQAL_GRAPH_PATTERN_OPERATOR_BASIC;
+    
+    /* update constraints expression after possible change */
+    rasqal_engine_build_constraints_expression(gp);
+
+  } else {
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG2("NOT OK to merge sub-graph patterns of %p\n", gp);
+#endif
+  }
+
+#if RASQAL_DEBUG > 1
+  if(merge_gp_ok) {
+    RASQAL_DEBUG2("Ending graph pattern %p\n  ", gp);
+    rasqal_graph_pattern_print(gp, stdout);
+    fputs("\n\n", stdout);
+  }
+#endif
+    
+}
