@@ -2,7 +2,7 @@
  *
  * rasqal_expr.c - Rasqal general expression support
  *
- * $Id: rasqal_expr.c,v 1.95 2005/03/20 12:22:12 cmdjb Exp $
+ * $Id: rasqal_expr.c,v 1.107 2005/06/08 16:20:25 cmdjb Exp $
  *
  * Copyright (C) 2003-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -102,25 +102,47 @@ rasqal_data_graph_print(rasqal_data_graph* dg, FILE* fh)
 
 
 /**
- * rasqal_new_variable - Constructor - Create a new Rasqal variable
+ * rasqal_new_variable_typed - Constructor - Create a new typed Rasqal variable
  * @rq: &rasqal_query to associate the variable with
+ * @type: variable type defined by enumeration rasqal_variable_type
  * @name: variable name
  * @value: variable &rasqal_literal value (or NULL)
  * 
  * The variable must be associated with a query, since variable
  * names are only significant with a single query.
  * 
+ * The @name and @value become owned by the rasqal_variable structure
+ *
  * Return value: a new &rasqal_variable or NULL on failure.
  **/
 rasqal_variable*
-rasqal_new_variable(rasqal_query* rq,
-                    const unsigned char *name, rasqal_literal *value) 
+rasqal_new_variable_typed(rasqal_query* rq,
+                          rasqal_variable_type type, 
+                          unsigned char *name, rasqal_literal *value)
 {
   int i;
   rasqal_variable* v;
+  raptor_sequence* seq;
+  int* count_p;
+
+  switch(type) {
+    case RASQAL_VARIABLE_TYPE_ANONYMOUS:
+      seq=rq->anon_variables_sequence;
+      count_p=&rq->anon_variables_count;
+      break;
+    case RASQAL_VARIABLE_TYPE_NORMAL:
+      seq=rq->variables_sequence;
+      count_p=&rq->variables_count;
+      break;
+
+    case RASQAL_VARIABLE_TYPE_UNKNOWN:
+    default:
+      RASQAL_DEBUG2("Unknown variable type %d", type);
+      return NULL;
+  }
   
-  for(i=0; i< raptor_sequence_size(rq->variables_sequence); i++) {
-    v=(rasqal_variable*)raptor_sequence_get_at(rq->variables_sequence, i);
+  for(i=0; i< raptor_sequence_size(seq); i++) {
+    v=(rasqal_variable*)raptor_sequence_get_at(seq, i);
     if(!strcmp((const char*)v->name, (const char*)name)) {
       /* name already present, do not need a copy */
       RASQAL_FREE(cstring, name);
@@ -130,16 +152,42 @@ rasqal_new_variable(rasqal_query* rq,
     
   v=(rasqal_variable*)RASQAL_CALLOC(rasqal_variable, sizeof(rasqal_variable), 1);
 
-  v->name=name;
-  v->value=value;
-  v->offset=rq->variables_count++;
+  v->type= type;
+  v->name= name;
+  v->value= value;
+  v->offset= (*count_p)++;
 
-  raptor_sequence_push(rq->variables_sequence, v);
+  raptor_sequence_push(seq, v);
   
   return v;
 }
 
 
+/**
+ * rasqal_new_variable - Constructor - Create a new Rasqal normal variable
+ * @rq: &rasqal_query to associate the variable with
+ * @name: variable name
+ * @value: variable &rasqal_literal value (or NULL)
+ * 
+ * The variable must be associated with a query, since variable
+ * names are only significant with a single query.
+ *
+ * This creates a regular variable that can be returned of type
+ * RASQAL_VARIABLE_TYPE_NORMAL.  Use rasqal_new_variable_typed
+ * to create other variables.
+ * 
+ * The @name and @value become owned by the rasqal_variable structure
+ *
+ * Return value: a new &rasqal_variable or NULL on failure.
+ **/
+rasqal_variable*
+rasqal_new_variable(rasqal_query* rq,
+                    unsigned char *name, rasqal_literal *value) 
+{
+  return rasqal_new_variable_typed(rq, RASQAL_VARIABLE_TYPE_NORMAL, name, value);
+}
+
+
 /**
  * rasqal_free_variable - Destructor - Destroy a Rasqal variable object
  * @v: &rasqal_variable object
@@ -149,7 +197,7 @@ void
 rasqal_free_variable(rasqal_variable* v)
 {
   if(v->name)
-    RASQAL_FREE(cstring, v->name);
+    RASQAL_FREE(cstring, (void*)v->name);
   if(v->value)
     rasqal_free_literal(v->value);
   RASQAL_FREE(rasqal_variable, v);
@@ -167,7 +215,10 @@ rasqal_free_variable(rasqal_variable* v)
 void
 rasqal_variable_print(rasqal_variable* v, FILE* fh)
 {
-  fprintf(fh, "variable(%s", v->name);
+  if(v->type == RASQAL_VARIABLE_TYPE_ANONYMOUS)
+    fprintf(fh, "anon-variable(%s", v->name);
+  else
+    fprintf(fh, "variable(%s", v->name);
   if(v->value) {
     fputc('=', fh);
     rasqal_literal_print(v->value, fh);
@@ -221,7 +272,7 @@ void
 rasqal_free_prefix(rasqal_prefix* p)
 {
   if(p->prefix)
-    RASQAL_FREE(cstring, p->prefix);
+    RASQAL_FREE(cstring, (void*)p->prefix);
   raptor_free_uri(p->uri);
   RASQAL_FREE(rasqal_prefix, p);
 }
@@ -271,13 +322,6 @@ rasqal_free_triple(rasqal_triple* t)
 }
 
 
-static const char *rasqal_triple_flag_strings[RASQAL_TRIPLE_FLAGS_LAST+1]={
-  NULL,
-  "EXACT",
-  "OPTIONAL"
-};
-
-
 void
 rasqal_triple_print(rasqal_triple* t, FILE* fh)
 {
@@ -293,24 +337,6 @@ rasqal_triple_print(rasqal_triple* t, FILE* fh)
     rasqal_literal_print(t->origin, fh);
     fputc(')', fh);
   }
-  if(t->flags) {
-    unsigned int f=t->flags;
-    unsigned int b=1;
-    int seen=0;
-    
-    fputs(" with flags(", fh);
-    while(f && b <= RASQAL_TRIPLE_FLAGS_LAST) {
-      if(seen)
-        fputs(", ", fh);
-      if(f & b) {
-        fputs(rasqal_triple_flag_strings[b], fh);
-        seen=1;
-        f &= ~b;
-      }
-      b <<= 1;
-    }
-    fputc(')', fh);
-  }
 }
 
 
@@ -328,20 +354,6 @@ rasqal_triple_get_origin(rasqal_triple* t)
 }
 
 
-void
-rasqal_triple_set_flags(rasqal_triple* t, unsigned int flags)
-{
-  t->flags=flags;
-}
-
-
-unsigned int
-rasqal_triple_get_flags(rasqal_triple* t)
-{
-  return t->flags;
-}
-
-
 rasqal_expression*
 rasqal_new_1op_expression(rasqal_op op, rasqal_expression* arg)
 {
@@ -440,6 +452,8 @@ rasqal_free_expression(rasqal_expression* e)
     case RASQAL_EXPR_ISURI:
     case RASQAL_EXPR_ISBLANK:
     case RASQAL_EXPR_ISLITERAL:
+    case RASQAL_EXPR_ORDER_COND_ASC:
+    case RASQAL_EXPR_ORDER_COND_DESC:
       rasqal_free_expression(e->arg1);
       break;
     case RASQAL_EXPR_STR_MATCH:
@@ -502,6 +516,8 @@ rasqal_expression_foreach(rasqal_expression* e,
     case RASQAL_EXPR_ISBLANK:
     case RASQAL_EXPR_ISLITERAL:
     case RASQAL_EXPR_CAST:
+    case RASQAL_EXPR_ORDER_COND_ASC:
+    case RASQAL_EXPR_ORDER_COND_DESC:
       return fn(user_data, e) ||
         rasqal_expression_foreach(e->arg1, fn, user_data);
       break;
@@ -802,6 +818,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
       {
         rasqal_literal* l;
         const unsigned char* s;
+        unsigned char* new_s;
+        size_t len;
         
         l=rasqal_expression_evaluate(query, e->arg1);
         if(!l)
@@ -813,7 +831,12 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
           goto failed;
         }
         
-        result=rasqal_new_string_literal(s, NULL, NULL, NULL);
+        len=strlen((const char*)s);
+
+        new_s=(unsigned char *)RASQAL_MALLOC(cstring, len+1);
+        strncpy((char*)new_s, (const char*)s, len+1);
+
+        result=rasqal_new_string_literal(new_s, NULL, NULL, NULL);
         rasqal_free_literal(l);
 
         break;
@@ -875,6 +898,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
           rasqal_free_literal(l);
           l=v->value;
           free_literal=0;
+          if(!l)
+            goto failed;
         }
 
         if(l->type != RASQAL_LITERAL_STRING) {
@@ -910,6 +935,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
           rasqal_free_literal(l);
           l=v->value;
           free_literal=0;
+          if(!l)
+            goto failed;
         }
 
         b=(l->type == RASQAL_LITERAL_URI);
@@ -937,6 +964,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
           rasqal_free_literal(l);
           l=v->value;
           free_literal=0;
+          if(!l)
+            goto failed;
         }
 
         b=(l->type == RASQAL_LITERAL_BLANK);
@@ -964,6 +993,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
           rasqal_free_literal(l);
           l=v->value;
           free_literal=0;
+          if(!l)
+            goto failed;
         }
 
         b=(l->type == RASQAL_LITERAL_STRING);
@@ -1282,7 +1313,7 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         rc= -1;
 #endif
 
-        RASQAL_DEBUG5("regex match returned %s for '%s' against '%s' (flags=%s)\n", b ? "true " : "false", match_string, pattern, l2->flags ? (char*)l2->flags : "");
+        RASQAL_DEBUG5("regex match returned %s for '%s' against '%s' (flags=%s)\n", b ? "true" : "false", match_string, pattern, l2->flags ? (char*)l2->flags : "");
         
         if(e->op == RASQAL_EXPR_STR_NMATCH)
           b=1-b;
@@ -1301,7 +1332,8 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
       break;
 
     case RASQAL_EXPR_FUNCTION:
-      RASQAL_FATAL1("No function expressions yet");
+      rasqal_query_warning(query, "No function expressions support at present.  Returning false.");
+      result=rasqal_new_boolean_literal(0);
       break;
       
     case RASQAL_EXPR_CAST:
@@ -1326,6 +1358,10 @@ rasqal_expression_evaluate(rasqal_query *query, rasqal_expression* e)
         break;
       }
 
+  case RASQAL_EXPR_ORDER_COND_ASC:
+  case RASQAL_EXPR_ORDER_COND_DESC:
+      result=rasqal_expression_evaluate(query, e->arg1);
+      break;
 
     case RASQAL_EXPR_UNKNOWN:
     default:
@@ -1378,7 +1414,9 @@ static const char* rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "isUri",
   "isBlank",
   "isLiteral",
-  "cast"
+  "cast",
+  "order asc",
+  "order desc"
 };
 
 void
@@ -1439,6 +1477,8 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_ISURI:
     case RASQAL_EXPR_ISBLANK:
     case RASQAL_EXPR_ISLITERAL:
+    case RASQAL_EXPR_ORDER_COND_ASC:
+    case RASQAL_EXPR_ORDER_COND_DESC:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
