@@ -2,7 +2,7 @@
  *
  * roqet.c - Rasqal RDF Query test program
  *
- * $Id: roqet.c,v 1.70 2005/03/10 00:05:20 cmdjb Exp $
+ * $Id: roqet.c,v 1.85 2005/06/08 16:33:20 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -68,9 +68,10 @@ static char *program=NULL;
 
 
 static enum {
-  OUTPUT_FORMAT_SIMPLE,
-  OUTPUT_FORMAT_XML
-} output_format = OUTPUT_FORMAT_SIMPLE;
+  RESULTS_FORMAT_SIMPLE,
+  RESULTS_FORMAT_XML_V1,
+  RESULTS_FORMAT_XML_V2
+} results_format = RESULTS_FORMAT_SIMPLE;
 
 
 int
@@ -90,7 +91,7 @@ rdql_parser_error(const char *msg)
 #endif
 
 
-#define GETOPT_STRING "cdf:ho:i:nqs:vw"
+#define GETOPT_STRING "cdhi:nr:qs:vw"
 
 #ifdef HAVE_GETOPT_LONG
 static struct option long_options[] =
@@ -99,11 +100,10 @@ static struct option long_options[] =
   {"count", 0, 0, 'c'},
   {"dump-query", 0, 0, 'd'},
   {"dryrun", 0, 0, 'n'},
-  {"format", 1, 0, 'f'},
   {"help", 0, 0, 'h'},
-  {"output", 1, 0, 'o'},
   {"input", 1, 0, 'i'},
   {"quiet", 0, 0, 'q'},
+  {"results", 1, 0, 'r'},
   {"source", 1, 0, 's'},
   {"version", 0, 0, 'v'},
   {"walk-query", 0, 0, 'w'},
@@ -127,7 +127,8 @@ static const char *title_format_string="Rasqal RDF query utility %s\n";
 
 static int
 roqet_query_results_print_as_xml(rasqal_query_results *results, FILE *fh,
-                                 raptor_uri* base_uri)
+                                 raptor_uri* base_uri,
+                                 int format)
 {
   raptor_iostream *iostr;
   raptor_uri* uri;
@@ -135,8 +136,12 @@ roqet_query_results_print_as_xml(rasqal_query_results *results, FILE *fh,
   iostr=raptor_new_iostream_to_file_handle(fh);
   if(!iostr)
     return 1;
-  
-  uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/");
+
+  if(format == RESULTS_FORMAT_XML_V1)
+    uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/");
+  else /* RESULTS_FORMAT_XML_V2 */
+    uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/sw/DataAccess/rf1/result2");
+
   rasqal_query_results_write(iostr, results, uri, base_uri);
 
   raptor_free_uri(uri);
@@ -161,89 +166,190 @@ roqet_error_handler(void *user_data,
 
 static const char *spaces="                                                                                  ";
 
+static void
+roqet_write_indent(FILE *fh, int indent) 
+{
+  if(indent > 0)
+    fwrite(spaces, sizeof(char), indent, fh);
+}
+
+  
+
 static void
 roqet_graph_pattern_walk(rasqal_graph_pattern *gp, int gp_index,
                          FILE *fh, int indent) {
   int triple_index=0;
-  int flags;
-      
-  flags=rasqal_graph_pattern_get_flags(gp);
+  rasqal_graph_pattern_operator op;
+  int seen;
+  raptor_sequence *seq;
   
-  fwrite(spaces, sizeof(char), indent, fh);
-  fprintf(fh, "graph pattern %d with flags %d\n", gp_index, flags);
+  op=rasqal_graph_pattern_get_operator(gp);
+  
+  roqet_write_indent(fh, indent);
+  fprintf(fh, "%s graph pattern", 
+          rasqal_graph_pattern_operator_as_string(op));
+  if(gp_index >= 0)
+    fprintf(fh, " #%d", gp_index);
+  fputs(" {\n", fh);
   
   indent+= 2;
 
+  /* look for triples */
+  seen=0;
   while(1) {
     rasqal_triple* t=rasqal_graph_pattern_get_triple(gp, triple_index);
     if(!t)
       break;
     
-    fwrite(spaces, sizeof(char), indent, fh);
-    fprintf(fh, "triple %d: ", triple_index);
+    if(!seen) {
+      roqet_write_indent(fh, indent);
+      fputs("triples {\n", fh);
+      seen=1;
+    }
+    roqet_write_indent(fh, indent+2);
+    fprintf(fh, "triple #%d { ", triple_index);
     rasqal_triple_print(t, fh);
-    fputc('\n', fh);
+    fputs(" }\n", fh);
 
     triple_index++;
   }
-
-  /* look for sub graph patterns */
-  gp_index=0;
-  while(1) {
-    rasqal_graph_pattern* sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
-    if(!sgp)
-      break;
-
-    roqet_graph_pattern_walk(sgp, gp_index, fh, indent);
-    gp_index++;
+  if(seen) {
+    roqet_write_indent(fh, indent);
+    fputs("}\n", fh);
   }
 
-  if(gp_index > 0) {
-    fwrite(spaces, sizeof(char), indent, fh);
-    fprintf(fh, "found %d sub-graph patterns\n", gp_index);
+
+  /* look for sub-graph patterns */
+  seq=rasqal_graph_pattern_get_sub_graph_pattern_sequence(gp);
+  if(seq && raptor_sequence_size(seq) > 0) {
+    roqet_write_indent(fh, indent);
+    fprintf(fh, "sub-graph patterns (%d) {\n", raptor_sequence_size(seq));
+
+    gp_index=0;
+    while(1) {
+      rasqal_graph_pattern* sgp=rasqal_graph_pattern_get_sub_graph_pattern(gp, gp_index);
+      if(!sgp)
+        break;
+      
+      roqet_graph_pattern_walk(sgp, gp_index, fh, indent+2);
+      gp_index++;
+    }
+
+    roqet_write_indent(fh, indent);
+    fputs("}\n", fh);
   }
+  
 
   /* look for constraints */
-  gp_index=0;
-  while(1) {
-    rasqal_expression* expr=rasqal_graph_pattern_get_constraint(gp, gp_index);
-    if(!expr)
-      break;
+  seq=rasqal_graph_pattern_get_constraint_sequence(gp);
+  if(seq && raptor_sequence_size(seq) > 0) {
+    roqet_write_indent(fh, indent);
+    fprintf(fh, "constraints (%d) {\n", raptor_sequence_size(seq));
 
-    fwrite(spaces, sizeof(char), indent, fh);
-    fprintf(fh, "constraint %d: ", triple_index);
-    rasqal_expression_print(expr, fh);
-    fputc('\n', fh);
+    gp_index=0;
+    while(1) {
+      rasqal_expression* expr=rasqal_graph_pattern_get_constraint(gp, gp_index);
+      if(!expr)
+        break;
+      
+      roqet_write_indent(fh, indent+2);
+      fprintf(fh, "constraint #%d { ", gp_index);
+      rasqal_expression_print(expr, fh);
+      fputs("}\n", fh);
+      
+      gp_index++;
+    }
 
-    gp_index++;
+    roqet_write_indent(fh, indent);
+    fputs("}\n", fh);
   }
+  
 
-  fputc('\n', fh);
-
+  indent-=2;
+  
+  roqet_write_indent(fh, indent);
+  fputs("}\n", fh);
 }
 
 
     
 static void
-roqet_query_walk(rasqal_query *rq, FILE *fh) {
-  int gp_index=0;
+roqet_query_walk(rasqal_query *rq, FILE *fh, int indent) {
+  rasqal_query_verb verb;
   int i;
+  rasqal_graph_pattern* gp;
+  raptor_sequence *seq;
+
+  verb=rasqal_query_get_verb(rq);
+  roqet_write_indent(fh, indent);
+  fprintf(fh, "query verb: %s\n", rasqal_query_verb_as_string(verb));
 
   i=rasqal_query_get_distinct(rq);
-  if(i != 0)
+  if(i != 0) {
+    roqet_write_indent(fh, indent);
     fprintf(fh, "query asks for distinct results\n");
-  i=rasqal_query_get_limit(rq);
-  if(i >= 0)
-    fprintf(fh, "query asks for result limits %d\n", i);
-  
-  while(1) {
-    rasqal_graph_pattern* gp=rasqal_query_get_graph_pattern(rq, gp_index);
-    if(!gp)
-      break;
-    
-    roqet_graph_pattern_walk(gp, gp_index, fh, 2);
-    gp_index++;
   }
+  
+  i=rasqal_query_get_limit(rq);
+  if(i >= 0) {
+    roqet_write_indent(fh, indent);
+    fprintf(fh, "query asks for result limits %d\n", i);
+  }
+  
+  i=rasqal_query_get_offset(rq);
+  if(i >= 0) {
+    roqet_write_indent(fh, indent);
+    fprintf(fh, "query asks for result offset %d\n", i);
+  }
+  
+  seq=rasqal_query_get_bound_variable_sequence(rq);
+  if(seq && raptor_sequence_size(seq) > 0) {
+    fprintf(fh, "query bound variables (%d): ", 
+            raptor_sequence_size(seq));
+    i=0;
+    while(1) {
+      rasqal_variable* v=(rasqal_variable*)raptor_sequence_get_at(seq, i);
+      if(!v)
+        break;
+
+      if(i > 0)
+        fputs(", ", fh);
+      fputs((const char*)v->name, fh);
+
+      i++;
+    }
+    fputc('\n', fh);
+  }
+
+  gp=rasqal_query_get_query_graph_pattern(rq);
+  if(!gp)
+    return;
+
+
+  seq=rasqal_query_get_construct_triples_sequence(rq);
+  if(seq && raptor_sequence_size(seq) > 0) {
+    roqet_write_indent(fh, indent);
+    fprintf(fh, "query construct triples (%d) {\n", 
+            raptor_sequence_size(seq));
+    i=0;
+    while(1) {
+      rasqal_triple* t=rasqal_query_get_construct_triple(rq, i);
+      if(!t)
+        break;
+    
+      roqet_write_indent(fh, indent+2);
+      fprintf(fh, "triple #%d { ", i);
+      rasqal_triple_print(t, fh);
+      fputs(" }\n", fh);
+
+      i++;
+    }
+    roqet_write_indent(fh, indent);
+    fputs("}\n", fh);
+  }
+
+  fputs("query ", fh);
+  roqet_graph_pattern_walk(gp, -1, fh, indent);
 }
 
 
@@ -257,7 +363,7 @@ main(int argc, char *argv[])
   unsigned char *base_uri_string=NULL;
   rasqal_query *rq;
   rasqal_query_results *results;
-  char *ql_name="rdql";
+  char *ql_name="sparql";
   char *ql_uri=NULL;
   int rc=0;
   raptor_uri *uri=NULL;
@@ -314,30 +420,6 @@ main(int argc, char *argv[])
         dump_query=1;
         break;
 
-      case 'f':
-        if(optarg) {
-          if(raptor_serializer_syntax_name_check(optarg))
-            serializer_syntax_name=optarg;
-          else {
-            int i;
-            
-            fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\n",
-                    program, optarg);
-            fprintf(stderr, "Valid arguments are:\n");
-            for(i=0; 1; i++) {
-              const char *help_name;
-              const char *help_label;
-              if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
-                break;
-              printf("  %-12s for %s\n", help_name, help_label);
-            }
-            usage=1;
-            break;
-            
-          }
-        }
-        break;
-
       case 'h':
         help=1;
         break;
@@ -346,17 +428,23 @@ main(int argc, char *argv[])
         dryrun=1;
         break;
 
-      case 'o':
+      case 'r':
         if(optarg) {
           if(!strcmp(optarg, "simple"))
-            output_format=OUTPUT_FORMAT_SIMPLE;
-          if(!strcmp(optarg, "xml"))
-            output_format=OUTPUT_FORMAT_XML;
+            results_format=RESULTS_FORMAT_SIMPLE;
+          else if(!strcmp(optarg, "xml"))
+            results_format=RESULTS_FORMAT_XML_V2;
+          else if(!strcmp(optarg, "xml-v1"))
+            results_format=RESULTS_FORMAT_XML_V1;
+          else if(raptor_serializer_syntax_name_check(optarg))
+            serializer_syntax_name=optarg;
           else {
-            fprintf(stderr, "%s: invalid argument `%s' for `" HELP_ARG(o, output) "'\n",
+            fprintf(stderr, 
+                    "%s: invalid argument `%s' for `" HELP_ARG(r, results) "'\n",
                     program, optarg);
-            fprintf(stderr, "Valid arguments are:\n  `simple'   for a simple format (default)\n  `xml'      for SPARQL variable bindings XML format\n");
             usage=1;
+            break;
+            
           }
         }
         break;
@@ -448,18 +536,6 @@ main(int argc, char *argv[])
     puts("Run an RDF query giving variable bindings or RDF triples.");
     puts("\nMain options:");
     puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
-    puts(HELP_TEXT("f FORMAT", "format FORMAT", "Set graph result format to one of:"));
-    for(i=0; 1; i++) {
-      const char *help_name;
-      const char *help_label;
-      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
-        break;
-      printf("    %-15s         %s", help_name, help_label);
-      if(!i)
-        puts(" (default)");
-      else
-        putchar('\n');
-    }
     puts(HELP_TEXT("i", "input LANGUAGE  ", "Set query language name to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
@@ -472,9 +548,23 @@ main(int argc, char *argv[])
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("o", "output FORMAT   ", "Set variable binding result format to one of:"));
-    puts("    simple                  A simple text format (default)");
-    puts("    xml                     SPARQL variable bindings XML format");
+    puts(HELP_TEXT("r", "results FORMAT  ", "Set query results format to one of:"));
+    puts("    For variable bindings and boolean results:");
+    puts("      simple                A simple text format (default)");
+    puts("      xml                   SPARQL Query Results XML format V2");
+    puts("      xml-v1                SPARQL Query Results XML format V1");
+    puts("    For RDF graph results:");
+    for(i=0; 1; i++) {
+      const char *help_name;
+      const char *help_label;
+      if(raptor_serializers_enumerate(i, &help_name, &help_label, NULL, NULL))
+        break;
+      printf("      %-15s       %s", help_name, help_label);
+      if(!i)
+        puts(" (default)");
+      else
+        putchar('\n');
+    }
     puts("\nAdditional options:");
     puts(HELP_TEXT("c", "count           ", "Count triples - no output"));
     puts(HELP_TEXT("d", "dump-query      ", "Dump the parsed query"));
@@ -482,8 +572,8 @@ main(int argc, char *argv[])
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("s", "source URI      ", "Query against RDF data at source URI"));
     puts(HELP_TEXT("v", "version         ", "Print the Rasqal version"));
-    puts(HELP_TEXT("d", "walk-query      ", "Walk the prepared query using the API"));
-    puts("\nReport bugs to <redland-dev@lists.librdf.org>.");
+    puts(HELP_TEXT("w", "walk-query      ", "Walk the prepared query using the API"));
+    puts("\nReport bugs to http://bugs.librdf.org/");
     puts("Rasqal home page: http://librdf.org/rasqal/");
     exit(0);
   }
@@ -628,7 +718,7 @@ main(int argc, char *argv[])
 
   if(walk_query) {
     fprintf(stderr, "%s: walking query structure\n", program);
-    roqet_query_walk(rq, stdout);
+    roqet_query_walk(rq, stdout, 0);
   }
   
   if(dump_query) {
@@ -645,40 +735,44 @@ main(int argc, char *argv[])
     goto tidy_query;
   }
 
-  if(output_format == OUTPUT_FORMAT_XML) {
-    roqet_query_results_print_as_xml(results, stdout, base_uri);
+  if(results_format == RESULTS_FORMAT_XML_V1 ||
+     results_format == RESULTS_FORMAT_XML_V2) {
+    roqet_query_results_print_as_xml(results, stdout, base_uri, 
+                                     results_format);
   } else {
     if(rasqal_query_results_is_bindings(results)) {
       if(!quiet)
-        fprintf(stdout, "%s: Query returned bindings results:\n", program);
+        fprintf(stdout, "%s: Query has a variable bindings result\n", program);
 
       while(!rasqal_query_results_finished(results)) {
-        int i;
-
-        fputs("result: [", stdout);
-        for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
-          const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
-          rasqal_literal *value=rasqal_query_results_get_binding_value(results, i);
-
-          if(i>0)
-            fputs(", ", stdout);
-          fprintf(stdout, "%s=", name);
-          if(value)
-            rasqal_literal_print(value, stdout);
-          else
-            fputs("NULL", stdout);
+        if(!count) {
+          int i;
+          
+          fputs("result: [", stdout);
+          for(i=0; i<rasqal_query_results_get_bindings_count(results); i++) {
+            const unsigned char *name=rasqal_query_results_get_binding_name(results, i);
+            rasqal_literal *value=rasqal_query_results_get_binding_value(results, i);
+            
+            if(i>0)
+              fputs(", ", stdout);
+            fprintf(stdout, "%s=", name);
+            if(value)
+              rasqal_literal_print(value, stdout);
+            else
+              fputs("NULL", stdout);
+          }
+          fputs("]\n", stdout);
         }
-        fputs("]\n", stdout);
-
+        
         rasqal_query_results_next(results);
       }
 
       if(!quiet)
-        fprintf(stderr, "%s: Query returned %d results\n", program, 
+        fprintf(stdout, "%s: Query returned %d results\n", program, 
                 rasqal_query_results_get_count(results));
 
     } else if (rasqal_query_results_is_boolean(results)) {
-      fprintf(stdout, "%s: Query returned boolean result: %s\n",
+      fprintf(stdout, "%s: Query has a boolean result: %s\n",
               program,
               rasqal_query_results_get_boolean(results) ? "true" : "false");
     }
@@ -686,7 +780,7 @@ main(int argc, char *argv[])
       int triple_count=0;
       
       if(!quiet)
-        fprintf(stdout, "%s: Query returned graph result:\n", program);
+        fprintf(stdout, "%s: Query has a graph result:\n", program);
 
       serializer=raptor_new_serializer(serializer_syntax_name);
       if(!serializer) {
